//-----------------------------------------------------------------------------
// File: CGameApp.h
//
// Desc: Game Application class, this is the central hub for all app processing
//
// Copyright (c) 1997-2002 Adam Hoult & Gary Simmons. All rights reserved.
//-----------------------------------------------------------------------------

#ifndef _CGAMEAPP_H_
#define _CGAMEAPP_H_

//-----------------------------------------------------------------------------
// CGameApp Specific Includes
//-----------------------------------------------------------------------------
#include "3D_Main.h"
#include "CTimer.h"
#include "CObject_DX.h"
#include "CPlayer.h"
#include "CTerrain.h"
#include "../../Source/CG/CG_Ocean.h"
#include "../../Source/Shader/Shader_Fog.h"
//-----------------------------------------------------------------------------
// Forward Declarations
//-----------------------------------------------------------------------------
namespace Main_Core
{
	 const float  WaterLevel  = 54.0f;               // Height of the water level (will be multiplied by Y scale)
}

class CCamera;



//-----------------------------------------------------------------------------
// Main Class Declarations
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Name : CGameApp (Class)
// Desc : Central game engine, initialises the game and handles core processes.
//-----------------------------------------------------------------------------
class CGameApp
{
public:
    //-------------------------------------------------------------------------
	// Constructors & Destructors for This Class.
	//-------------------------------------------------------------------------
	         CGameApp();
	virtual ~CGameApp();

	//-------------------------------------------------------------------------
	// App State Access Functions
	//-------------------------------------------------------------------------
    const D3DTEXTUREOP & GetColorOp() const { return m_ColorOp; }

	//-------------------------------------------------------------------------
	// Public Functions for This Class
	//-------------------------------------------------------------------------
 //   LRESULT     DisplayWndProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam );
	bool        InitInstance(HWND hwnd, CRect rc);
    void        FrameAdvance();
	bool        ShutDown();
	void		SetSize(bool isMin, CRect rc);
	void		Mouse_Lift_Down();
	void		Mouse_Lift_Up();
	void		Camera_Mode_Change(int camera_num);
	void        SetupRenderStates ( );
	void		Change_Direct3D();
	void		Change_Terrian(bool SinglePass);
	void		SelectMenuItems   ( );

private:
    //-------------------------------------------------------------------------
	// Private Functions for This Class
	//-------------------------------------------------------------------------
    bool        BuildObjects      ( );
	void        ReleaseObjects    ( );
	void        ChangeDevice      ( );
    void        SetupGameState    ( );
    

    void        AnimateObjects    ( );
    void        ProcessInput      ( );
    bool        InitDirect3D      (CRect rc);

	bool        BuildSkyBox       ( );
    void        RenderSkyBox      ( );
	bool		BuildOcean		  (float Width, float Height);
	void		RenderOcean		  ( );

    D3DFORMAT   FindDepthStencilFormat( ULONG AdapterOrdinal, D3DDISPLAYMODE Mode, D3DDEVTYPE DevType );
    
    //-------------------------------------------------------------------------
	// Private Variables For This Class
	//-------------------------------------------------------------------------
	CTerrain                m_Terrain;          // Simple terrain object (stores data)
    
    CPlayer                 m_Player;           // Player class used to manipulate our player object
    CCamera               * m_pCamera;          // A cached copy of the CCamera attached to the player

    D3DXMATRIXA16           m_mtxIdentity;      // A basic identity matrix
    
    CTimer                  m_Timer;            // Game timer
    ULONG                   m_LastFrameRate;    // Used for making sure we update only when fps changes.
    
    HWND                    m_hWnd;             // Main window HWND
    HICON                   m_hIcon;            // Window Icon
    HMENU                   m_hMenu;            // Window Menu
    
    bool                    m_bLostDevice;      // Is the 3d device currently lost ?
    bool                    m_bActive;          // Is the application active ?

    LPDIRECT3D9             m_pD3D;             // Direct3D Object
    LPDIRECT3DDEVICE9       m_pD3DDevice;       // Direct3D Device Object
    D3DPRESENT_PARAMETERS   m_D3DPresentParams; // Direct3D Present Parameters

    bool                    m_MagFilterCaps[10];// Capabilities supported for our required filters
    bool                    m_MinFilterCaps[10];// Capabilities supported for our required filters
    bool                    m_MipFilterCaps[10];// Capabilities supported for our required mip filters
    bool                    m_ColorOpCaps[30];  // Capabilities supported for our required color ops
    ULONG                   m_MaxTextures;      // Capabilities supported for our required pass count
    ULONG                   m_MaxAnisotropy;    // Capabilities supported for anisotropy filter

    ULONG                   m_nViewX;           // X Position of render viewport
    ULONG                   m_nViewY;           // Y Position of render viewport
    ULONG                   m_nViewWidth;       // Width of render viewport
    ULONG                   m_nViewHeight;      // Height of render viewport

    POINT                   m_OldCursorPos;     // Old cursor position for tracking

    CObject_DX              m_Object;           // The object referencing the player mesh
    CMesh                   m_PlayerMesh;       // The player mesh (cube ;)
    
    CMesh                   m_SkyMesh;          // The skybox mesh (also a cube)
    LPDIRECT3DTEXTURE9      m_SkyTextures[6];   // The skybox textures
	int						m_CameraMode;
	CG_Ocean				m_Ocean;
	Shader_Fog				m_Fog;

	// MFC ɼ  ϰ  .
public:
	D3DFILLMODE             m_FillMode;         // Which fill mode are we using ?
    D3DTEXTUREOP            m_ColorOp;          // Which color op are we using?
    D3DTEXTUREFILTERTYPE    m_MagFilter;        // Magnification Filter to use.
    D3DTEXTUREFILTERTYPE    m_MinFilter;        // Minification Filter to use.
    D3DTEXTUREFILTERTYPE    m_MipFilter;        // Mip-Map filter to use.
    ULONG                   m_Anisotropy;       // Anisotropy level to use.
    bool                    m_bSinglePass;      // Use single pass rendering method (multiple stage)
	
};

#endif // _CGAMEAPP_H_